/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.framework.server;

import org.apache.commons.lang.builder.ToStringBuilder;


/**
 * Represents a synchronization timestamp for a particular database and principal.
 * <i>tag</i> stores the next (when used as a next timestamp) or the last (when 
 * used as a last timstamp) value as it appears in <i>SyncAnchor</i>.
 *
 * @author  Stefano Fornari @ Funambol
 * @version $Id: SyncTimestamp.java,v 1.3.4.2 2005/01/20 16:09:23 luigiafassina Exp $
 */
public class SyncTimestamp {
    
    // ------------------------------------------------------------- Public data
    
    public long   start    = -1  ;  // timestamp of when sync started
    public long   end      = -1  ;  // timestamp of when sync ended
    public String tagServer = null;  // the last/next tag of the server
    public String tagClient = null;  // the last/next tag of the client
    
    // ------------------------------------------------------------ Constructors
    
    /** Creates a new instance of LastTimestamp */
    public SyncTimestamp() { }
    
    /** 
     * Creates a new instance of LastTimestamp when the only starting timestamp 
     * is known. <i>end</i> is set to -1 and tag is set to the string value
     * of the start.
     *
     * @param start the starting timestamp
     */
    public SyncTimestamp(final long start) {
        this(String.valueOf(start), String.valueOf(start), start, -1);
    }
    
    /** 
     * Creates a new instance of LastTimestamp.
     * It is need to cache the next anchor of the server and the next anchor 
     * of the client.
     *
     * @param tagServer the last/next tag of the server
     * @param tagClient the last/next tag of the client
     * @param start the timestamp of when sync started
     * @param end the timestamo of when sync ended
     *
     */
    public SyncTimestamp(final String tagServer, 
                         final String tagClient, 
                         final long   start, 
                         final long   end  ) {
        this.tagServer = tagServer;
        this.tagClient = tagClient;
        this.start    = start    ;
        this.end      = end      ;
    }
    
    // ---------------------------------------------------------- Public methods
    
    public String toString() {
        ToStringBuilder sb = new ToStringBuilder(this);
        
        sb.append("tagServer", tagServer).
           append("tagClient", tagClient).
           append("start"    , new java.sql.Timestamp(start)).
           append("end"      , new java.sql.Timestamp(end));
        return sb.toString();
    }
}